package com.iformall.mapper;

import java.util.*;
import com.iformall.common.CommonMapper;
import com.iformall.domain.po.WxMerchantTradeDaily;
import com.iformall.domain.vo.WxMerchantTradeDailyVo;

public interface WxMerchantTradeDailyMapper extends CommonMapper<WxMerchantTradeDaily, String> {

	List<WxMerchantTradeDaily> findList(WxMerchantTradeDaily wxMerchantTradeDaily);

	List<WxMerchantTradeDaily> findListOfWeek(WxMerchantTradeDaily wxMerchantTradeDaily);

	List<WxMerchantTradeDailyVo> findListVo(WxMerchantTradeDailyVo wxMerchantTradeDaily);

	List<WxMerchantTradeDailyVo> findListVoOfMonth(WxMerchantTradeDaily wxMerchantTradeDaily);

	List<WxMerchantTradeDaily> findTotalListOfMonth(WxMerchantTradeDaily wxMerchantTradeDaily);
}
